﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Life
{
    class Plansza
    {
        private int szerokosc = 20;
        public int Szerokosc 
        {
            get { return szerokosc; }
        }

        public int Wysokosc { get; private set; }

        private int liczbaZywych = 0;
        public int LiczbaZywych
        {
            get { return liczbaZywych; }
        }

        private char symbol = '*';
        public char Symbol
        {
            get { return symbol; }
            set { symbol = value; }
        }
        private int [,] plansza = null;
        private int numerCyklu = 0;
        private double prawdopodobienstwo = 0.33;
        ConsoleColor kolorTla = ConsoleColor.Green;
        ConsoleColor kolorZnakow = ConsoleColor.Red;



        public Plansza():this(20,20,0.33)
        { 
        }

        public Plansza(int wysokosc, int szerokosc, double prawdopodobienstwo)
        {
            Wysokosc = wysokosc;
            this.szerokosc = szerokosc;
            plansza = new int[szerokosc, wysokosc];
            Inicjalizacja(prawdopodobienstwo);
            UstawOkno();
        }

        public Plansza(int wysokosc, int szerokosc, double prawdopodobienstwo,char symbol,ConsoleColor kolor,ConsoleColor tlo)
        {
            
            this.Wysokosc = wysokosc;
            this.szerokosc = szerokosc;
            plansza = new int[Szerokosc, wysokosc];
            Inicjalizacja(prawdopodobienstwo);
            this.symbol = symbol;
            kolorTla = tlo;
            kolorZnakow = kolor;
            
            UstawOkno();
        
        }

        private void UstawOkno()
        {
            // Ustawienia okna i bufora zwieksza rozmiary jezeli za malo miejsca
            Console.Title = "Gra Życie";

            int xoffset = 10;
            int yoffset = 20;


            if (Console.BufferWidth < szerokosc + xoffset) Console.BufferWidth = szerokosc + xoffset;
            if (Console.BufferHeight < Wysokosc + yoffset) Console.BufferHeight = Wysokosc + yoffset;


            if (Console.WindowHeight < Wysokosc + yoffset)
                if (Wysokosc + yoffset < Console.LargestWindowHeight) Console.WindowHeight = Wysokosc + yoffset;
                else Console.WindowHeight = Console.LargestWindowHeight;
            if (Console.WindowWidth < Szerokosc + xoffset)
                if (szerokosc + xoffset < Console.LargestWindowWidth) Console.WindowWidth = Szerokosc + yoffset;
                else Console.WindowWidth = Console.LargestWindowWidth;
        }

        public void Inicjalizacja(double prawdopodobienstwo)
        {
            this.prawdopodobienstwo = prawdopodobienstwo;
            System.Random rnd = new Random();
            liczbaZywych = 0;
            for(int i=0;i<Szerokosc;i++)
                for (int j = 0; j < Wysokosc; j++)
                {
                    if (rnd.NextDouble() < prawdopodobienstwo)
                    {
                        plansza[i, j] = 1;
                        liczbaZywych++;
                    }
                    else plansza[i, j] = 0;
                }
            numerCyklu = 0;
        }

        public void Rysuj()
        {


            Console.ResetColor();
            Console.SetCursorPosition(0, 0);
            Console.WriteLine("Liczba żywych      : {0} / {1} [{2:F1}%]             ", liczbaZywych,szerokosc*Wysokosc,100.0*(double)liczbaZywych/(double)(szerokosc*Wysokosc));
            Console.WriteLine("Cykl               : {0}                             ", numerCyklu);
            Console.WriteLine("Wymiary            : {0}x{1}                         ", Szerokosc,Wysokosc);
            Console.WriteLine("prawdopodobienstwo : {0}                             ", prawdopodobienstwo);

            int startx = 3;
            int starty = 10;

            Console.ForegroundColor = kolorZnakow;
            Console.BackgroundColor = kolorTla;
            for (int i = 0; i < Szerokosc; i++)
                for (int j = 0; j < Wysokosc; j++)
                {
                    Console.SetCursorPosition(startx+i,starty+j);
                    if (plansza[i, j] == 1) Console.Write(symbol);
                    else Console.Write(' ');
                }

            Console.ResetColor();
            Console.WriteLine("\n");
        }

        private int LiczbaSasiadow(int x, int y)
        {
            int ls = 0;
            ls += plansza[(x + szerokosc - 1) % szerokosc, (y + Wysokosc - 1) % Wysokosc]%2;
            ls += plansza[(x + szerokosc - 1) % szerokosc, (y + Wysokosc) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc - 1) % szerokosc, (y + Wysokosc + 1) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc) % szerokosc, (y + Wysokosc - 1) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc) % szerokosc, (y + Wysokosc + 1) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc + 1) % szerokosc, (y + Wysokosc - 1) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc + 1) % szerokosc, (y + Wysokosc) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc + 1) % szerokosc, (y + Wysokosc + 1) % Wysokosc] % 2;

            return ls;
        }

        public void Cykl()
        {
            int n = liczbaZywych;
            for (int i = 0; i < Szerokosc; i++)
            {
                
                for (int j = 0; j < Wysokosc; j++)
                {
                    int ls=LiczbaSasiadow(i, j);
                    if (plansza[i, j] % 2 == 0 && ls == 3)
                    {
                        plansza[i, j] = 2;            // narodziny  
                        n++;
                    }
                    if (plansza[i, j] % 2 == 1 && (ls < 2 || ls > 3))
                    {
                        n--;
                        plansza[i, j] = 3;    // śmierć
                    }
                }
            }
            for (int i = 0; i < Szerokosc; i++)
                for (int j = 0; j < Wysokosc; j++)
                {
                    if (plansza[i, j] == 2) plansza[i, j] = 1;
                    if (plansza[i, j] == 3) plansza[i, j] = 0;
                }
            numerCyklu++;
            liczbaZywych = n;
        }

        public bool Zapisz(string nazwapliku)
        {
            using (StreamWriter plik = new StreamWriter(nazwapliku))
            {
                plik.WriteLine("Liczba żywych      : {0} / {1} [{2:F1}%]             ", liczbaZywych, szerokosc * Wysokosc, 100.0 * (double)liczbaZywych / (double)(szerokosc * Wysokosc));
                plik.WriteLine("Cykl               : {0}                             ", numerCyklu);
                plik.WriteLine("Wymiary            : {0}x{1}                         ", Szerokosc, Wysokosc);
                plik.WriteLine("prawdopodobienstwo : {0}                             ", prawdopodobienstwo);
                for (int i = 0; i < Szerokosc; i++)
                {
                    for (int j = 0; j < Wysokosc; j++)
                    {
                        if (plansza[i, j] == 1) plik.Write(symbol);
                        else plik.Write(' ');
                    }
                    plik.WriteLine();
                }
            }
            return true;
        }
        
    }
}
